from plone.rfc822.interfaces import IPrimaryFieldInfo
from collective.mediaelementjs.browser.view import File as FileViewBase


class MediaElementJSFileView(FileViewBase):
    def getFilename(self):
        info = IPrimaryFieldInfo(self.context)
        return info.value.filename



from Acquisition import aq_inner
import os

from Products.Five.browser import BrowserView

from collective.mediaelementjs.interfaces import IMediaInfo, IAudio, IVideo


class File(BrowserView):
    def __init__(self, context, request):
        self.context = context
        self.request = request

        self.isVideo = None
        self.isAudio = None

    def __call__(self):
        self.update()
        return self.index()

    def update(self):
        if self.isAudio is None:
            self.isAudio = IAudio.providedBy(self.context)
        if self.isVideo is None:
            self.isVideo = IVideo.providedBy(self.context)

    def video(self):
        if not self.isVideo:
            return
        info = IMediaInfo(self.context)
        return dict(
            url=self.href(),
            title=self.context.Title(),
            description=self.context.Description(),
            height=info.height,
            width=info.width,
            duration=info.duration
        )

    def audio(self):
        if not self.isAudio:
            return
        info = IMediaInfo(self.context)
        return dict(
            url=self.href(),
            title=self.context.Title(),
            description=self.context.Description(),
            duration=info.duration
        )

    def getFilename(self):
        context = aq_inner(self.context)
        return context.getFilename()

    def getContentType(self):
        return self.context.getContentType()

    def href(self):
        context = aq_inner(self.context)
        ext = ''
        url = context.absolute_url()
        filename = self.getFilename()
        if filename:
            extension = os.path.splitext(filename)[1]
            if not url.endswith(extension):
                ext = "?e=%s" % extension
        return self.context.absolute_url() + ext


class DXFile(BrowserView):
    def __init__(self, context, request):
        self.context = context
        self.request = request

        self.isVideo = None
        self.isAudio = None

    def __call__(self):
        self.update()
        return self.index()

    def update(self):
        if self.isAudio is None:
            self.isAudio = IAudio.providedBy(self.context)
        if self.isVideo is None:
            self.isVideo = IVideo.providedBy(self.context)

    def video(self):
        if not self.isVideo:
            return
        info = IMediaInfo(self.context)
        return dict(
            url=self.href(),
            title=self.context.Title(),
            description=self.context.Description(),
            height=info.height,
            width=info.width,
            duration=info.duration
        )

    def audio(self):
        if not self.isAudio:
            return
        info = IMediaInfo(self.context)
        return dict(
            url=self.href(),
            title=self.context.Title(),
            description=self.context.Description(),
            duration=info.duration
        )

    def filename(self):
        return self.context.file.filename

    def contentType(self):
        return self.context.file.contentType

    def href(self):
        context = aq_inner(self.context)
        ext = ''
        url = context.absolute_url()
        filename = self.filename()
        if filename:
            extension = os.path.splitext(filename)[1]
            if not url.endswith(extension):
                ext = "?e=%s" % extension
        return self.context.absolute_url() + ext



from zope.interface import Interface
from zope import schema

from collective.mediaelementjs import MessageFactory as _


class IMediaElementJSPlayable(Interface):
    """A file playable with mediaelementjs
    """


class IVideo(IMediaElementJSPlayable):
    """Marker interface for files that contain video content
    """


class IMediaInfo(Interface):
    """Information about a video object
    """
    width = schema.Int(title=_(u"Width"), required=False)
    height = schema.Int(title=_(u"Height"), required=False)
    duration = schema.Timedelta(title=_(u"Duration"), required=False)


class IAudio(IMediaElementJSPlayable):
    """Marker interface for files that contain audio content
    """



from persistent import Persistent

from zope.annotation import factory
from zope.component import adapts
from zope.interface import implements

from collective.mediaelementjs.interfaces import IVideo, IMediaInfo, IAudio


class VideoInfo(Persistent):
    implements(IMediaInfo)
    adapts(IVideo)

    def __init__(self):
        self.height = None
        self.width = None
        self.duration = None

VideoInfoAdapter = factory(VideoInfo)


class AudioInfo(Persistent):
    implements(IMediaInfo)
    adapts(IAudio)

    def __init__(self):
        self.duration = None

AudioInfoAdapter = factory(AudioInfo)



from setuptools import setup, find_packages
import os

version = '0.4.3'

tests_require = [
    'collective.testcaselayer',
    'interlude',
    'Products.PloneTestCase',
]

setup(name='collective.mediaelementjs',
      version=version,
      description="A simple integration of the MediaElementJS video player for Plone.",
      long_description=open("README.rst").read() + "\n" +
      open(os.path.join("docs", "HISTORY.rst")).read(),
      # Get more strings from
      # http://pypi.python.org/pypi?:action=list_classifiers
      classifiers=[
          "Environment :: Web Environment",
          "License :: OSI Approved :: GNU General Public License (GPL)",
          "Operating System :: OS Independent",
          "Framework :: Zope2",
          "Framework :: Plone",
          "Framework :: Plone :: 4.0",
          "Framework :: Plone :: 4.1",
          "Framework :: Plone :: 4.2",
          "Framework :: Plone :: 4.3",
          "Programming Language :: Python",
          "Programming Language :: Python :: 2.6",
          "Programming Language :: Python :: 2.7",
          "Topic :: Multimedia :: Video :: Display",
          "Topic :: Multimedia :: Sound/Audio :: Players",
          "Topic :: Multimedia :: Sound/Audio :: Players :: MP3",
      ],
      keywords='plone html5 audio video mp3 wma mp4 webm ogg flv wmv mpeg quicktime',
      author='Tom Lazar, Unweb.me',
      author_email='tom@tomster.org, we@unweb.me',
      maintainer='Servilio Afre Puentes',
      maintainer_email='afrepues@mcmaster.ca',
      url='https://github.com/collective/collective.mediaelementjs',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['collective'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'hachoir_core',
          'hachoir_metadata',
          'hachoir_parser',
          'Acquisition',
          'Products.Archetypes',
          'Products.ATContentTypes',
          'Products.CMFCore',
          'Products.GenericSetup >=1.6',
          'ZODB3',
          'Zope2',
          'plone.app.jquery',
          'plone.rfc822',
          'zope.annotation',
          'zope.cachedescriptors',
          'zope.component',
          'zope.i18nmessageid',
          'zope.interface',
          'zope.schema',
      ],
      tests_require=tests_require,
      extras_require={'test': tests_require},
      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



from zope.cachedescriptors import property
from zope.interface import alsoProvides, noLongerProvides

from collective.mediaelementjs.interfaces import IVideo, IMediaInfo, IAudio
from collective.mediaelementjs.metadata_extraction import parse_raw
from collective.mediaelementjs.metadata_extraction import defensive_get

from Products.ATContentTypes.interfaces.file import IFileContent
from Products.Archetypes.interfaces import IObjectInitializedEvent

try:
    from plone.app.contenttypes.interfaces import IFile
except ImportError:
    from zope.interface import Interface as IFile


from StringIO import StringIO


VIDEO_EXTENSIONS = [
    '.mp4', '.m4v',
    '.ogg', '.ogv',
    '.webm',
    '.flv',
    '.wmv',
    '.mpeg', '.mpg', '.mpe',
    '.mov', '.qt',
]

AUDIO_EXTENSIONS = [
    '.mp3',
    '.ogg', '.oga',
    '.m4a',
    '.wav', '.wma', '.webma'
]


def remove_marker(object):
    changed = False
    if IVideo.providedBy(object):
        noLongerProvides(object, IVideo)
        changed = True
    if IAudio.providedBy(object):
        noLongerProvides(object, IAudio)
        changed = True
    if changed:
        object.reindexObject(idxs=['object_provides'])


class ChangeView(object):

    interface = None
    value = None
    file_handle = None

    def __init__(self, object, event):
        self.object = object
        # TODO: do we really need this different from object?
        self.content = event.object
        if not self.interface.providedBy(self.content):
            return
        if self.value is None:
            remove_marker(self.content)
            return

        ext = self.check_extension()
        if ext is None:
            remove_marker(self.content)
            return

        # set the view to mediaelementjs view
        if IObjectInitializedEvent.providedBy(event):
            self.content.setLayout('mediaelementjs')

        if ext in VIDEO_EXTENSIONS:
            self.handleVideo()

        if ext in AUDIO_EXTENSIONS:
            self.handleAudio()

    def check_extension(self):
        for ext in VIDEO_EXTENSIONS:
            if self.filename.endswith(ext):
                return ext
        for ext in AUDIO_EXTENSIONS:
            if self.filename.endswith(ext):
                return ext
        return None

    def handleVideo(self):
        handle = self.file_handle
        metadata = parse_raw(handle)
        handle.close()

        if not IVideo.providedBy(self.content):
            alsoProvides(self.content, IVideo)
            self.object.reindexObject(idxs=['object_provides'])

        info = IMediaInfo(self.content)
        info.height = defensive_get(metadata, 'height')
        info.width = defensive_get(metadata, 'width')
        info.duration = defensive_get(metadata, 'duration')

    def handleAudio(self):
        handle = self.file_handle
        metadata = parse_raw(handle)
        handle.close()

        if not IAudio.providedBy(self.content):
            alsoProvides(self.content, IAudio)
            self.object.reindexObject(idxs=['object_provides'])

        info = IMediaInfo(self.content)
        info.height = defensive_get(metadata, 'height')
        info.width = defensive_get(metadata, 'width')
        info.duration = defensive_get(metadata, 'duration')


class ChangeFileView(ChangeView):

    interface = IFileContent

    @property.Lazy
    def value(self):
        return self.content.getField('file').getRaw(self.content)

    @property.Lazy
    def filename(self):
        filename = self.value.filename
        if isinstance(filename, basestring):
            filename = filename.lower()
        return filename

    @property.Lazy
    def file_handle(self):
        file_object = self.value
        try:
            # For blobs
            file_handle = file_object.getIterator()
        except AttributeError:
            file_handle = StringIO(str(file_object.data))
        return file_handle


class ChangeDXFileView(ChangeView):

    interface = IFile

    @property.Lazy
    def value(self):
        return self.content.file

    @property.Lazy
    def filename(self):
        filename = self.value.filename
        if isinstance(filename, basestring):
            filename = filename.lower()
        return filename

    @property.Lazy
    def file_handle(self):
        file_object = self.value
        try:
            # For blobs
            file_handle = file_object.getIterator()
        except AttributeError:
            file_handle = StringIO(str(file_object.data))
        return file_handle
